package jp.android.booksample.PendingIntentSample;

import java.util.Date;

import jp.android.booksample.PendingIntentSample.PendingIntentSampleFragment.PendingIntentSampleFragmentClickListener;
import android.app.Activity;
import android.app.AlarmManager;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.util.Log;
import android.widget.TextView;

public class PendingIntentSampleActivity extends Activity implements PendingIntent.OnFinished, PendingIntentSampleFragmentClickListener {
	public static final String EXTRA_KEY_MESSAGE = "MESSAGE";
	public static final String EXTRA_KEY_CALLING_TASK_ID = "CALLING_TASK_ID";
	public static final String INTENT_ACTION_SHOW_MESSAGE = "jp.android.booksample.PendingIntentSample.SHOW_MESSAGE";
	public static final String INTENT_ACTION_CANCEL_NOTIFY = "jp.android.booksample.PendingIntentSample.CANCEL_NOTIFY";
	
	private int mCallingTaskId = -1;
	
    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void onResume() {
    	Log.d(getLocalClassName(), "onResume()");
    	super.onResume();

    	// 呼び出し元からのメッセージがあれば表示する。
        Intent intent = getIntent();
        if (intent != null) {
	        String message = intent.getStringExtra(EXTRA_KEY_MESSAGE);
            TextView tv = (TextView) findViewById(R.id.message_text);
	        if (message != null) {
	            tv.setText(message);        	
	        }
	        else {
	            tv.setText("");        	
	        }
	        // Intentから呼び出し元タスクIDを取得する。
	        mCallingTaskId = intent.getIntExtra(EXTRA_KEY_CALLING_TASK_ID, -1);

	        String action = intent.getAction();
	        if (action != null) {
		        if (action.equals(INTENT_ACTION_CANCEL_NOTIFY)) {
		            NotificationManager nm = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		            nm.cancel(1);
		        }
	        }
	    }
    	
        // タスクIDを表示する。
    	PendingIntentSampleFragment fragment = (PendingIntentSampleFragment) getFragmentManager().findFragmentById(R.id.frag_pending_intent);
    	if (fragment != null) {
    		fragment.setTaskId(getTaskId());
    		if (mCallingTaskId != -1) {
    			fragment.setCallingTaskId(mCallingTaskId);
    		}
    	}
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void onStart() {
    	Log.d(getLocalClassName(), "onStart()");
    	super.onStart();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void onStop() {
    	Log.d(getLocalClassName(), "onStop()");
    	super.onStop();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void onPause() {
    	Log.d(getLocalClassName(), "onPause()");
    	super.onPause();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void onDestroy() {
    	Log.d(getLocalClassName(), "onDestroy()");
    	super.onDestroy();
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void onActivityResult (int requestCode, int resultCode, Intent data) {
		Log.d(getLocalClassName(), "onActivityResult()");
    	if (resultCode != Activity.RESULT_CANCELED) {
    	}
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    protected void onNewIntent(Intent intent) {
		Log.d(getLocalClassName(), "onNewIntent()");
        setIntent(intent);
        super.onNewIntent(intent);
    }

    /**
     * 送信完了のハンドラ
     */
	@Override
	public void onSendFinished(PendingIntent pendingIntent, Intent intent,	int resultCode, String resultData, Bundle resultExtras) {
		Log.d(getLocalClassName(), "onSendFinished(). resultCode: " + resultCode + " resultData: " + resultData);
	}
	
	/**
	 * Dial、Mail、Map、Browserアプリを連続実行するためのIntent配列を作成する。
	 * @return Activityスタックを作成するためのIntent配列
	 */
	static Intent[] makeIntentStack() {
		Intent[] intents = new Intent[4];
		// Dial
		intents[0] = new Intent(Intent.ACTION_VIEW, Uri.parse("tel:012-345-6789"));
		intents[0].setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		// Mail
		intents[1] = new Intent(Intent.ACTION_SENDTO, Uri.parse("mailto:xxxx@xxxx.com"));
		intents[1].putExtra(Intent.EXTRA_SUBJECT, "メールの件名");
		intents[1].putExtra(Intent.EXTRA_TEXT, "メールの本文");
		// Map
		intents[2] = new Intent(Intent.ACTION_VIEW, Uri.parse("geo:0,0?q=Tokyo"));
		// Browser
		intents[3] = new Intent(Intent.ACTION_VIEW, Uri.parse("http://www.google.com"));

	    return intents;
	}

	/**
	 * [Activities]ボタンハンドラ
	 * 複数のActivityに対して、PendingIntentを送信する。
	 */
	@Override
	public void onClickActivitiesButton() {
    	Context appContext = getApplicationContext();
    	
    	// 複数のActivityを実行するPendingIntentの作成
    	PendingIntent sender = PendingIntent.getActivities(appContext, 1, makeIntentStack(), PendingIntent.FLAG_ONE_SHOT);
    	try {
    		sender.send(appContext, 1, null, this, null, null);
    	}
    	catch (PendingIntent.CanceledException e) {
    		Log.d(getLocalClassName(), e.toString());
    	}	
    }

	/**
	 * [Service]ボタンハンドラ
	 * PendingIntentをサービス宛てに送る。
	 */
	@Override
	public void onClickServiceButton() {
    	Context appContext = getApplicationContext();
    	
    	// PendingIntentの作成
    	Intent intent = new Intent(appContext, jp.android.booksample.PendingIntentSample.PendingIntentSampleService.class);
    	intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    	intent.putExtra(EXTRA_KEY_MESSAGE, "called " + getLocalClassName() + " passed Service");
    	intent.putExtra(EXTRA_KEY_CALLING_TASK_ID, getTaskId());
    	intent.setAction(INTENT_ACTION_SHOW_MESSAGE);
    	
    	// サービス宛てのPendingIntentの作成
    	PendingIntent sender = PendingIntent.getService(appContext, 1, intent, PendingIntent.FLAG_ONE_SHOT);
    	// AlramManager取得
    	AlarmManager am = (AlarmManager) getSystemService(ALARM_SERVICE);
    	am.set(AlarmManager.RTC_WAKEUP, System.currentTimeMillis() + 3000, sender);

    }

	/**
	 * [Broadcast]ボタンハンドラ
	 * PendingIntentをブロードキャストして、自Activityを終了する。
	 */
	@Override
	public void onClickBroadcastButton() {
		long curTime = System.currentTimeMillis();
    	Context appContext = getApplicationContext();
    	// Receiverを呼び出すインテントを作成
    	Intent intent = new Intent(appContext, PendingIntentSampleReceiver.class);
    	intent.putExtra(EXTRA_KEY_MESSAGE, "called " + getLocalClassName() + " at " + new Date(curTime));
    	intent.setAction(INTENT_ACTION_SHOW_MESSAGE);

    	// ブロードキャストを投げるPendingIntentの作成
    	PendingIntent sender = PendingIntent.getBroadcast(appContext, 0, intent, 0);

    	// AlramManager取得
    	AlarmManager am = (AlarmManager) getSystemService(ALARM_SERVICE);
    	am.set(AlarmManager.RTC_WAKEUP, curTime + 5000, sender);

    	finish();
	}

}