package jp.android.booksample.PendingIntentSample;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Intent;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Binder;
import android.os.Bundle;
import android.os.IBinder;
import android.util.Log;
import android.widget.Toast;

/**
 * 通知を経由してActivityを起動するサービス。
 * @author Makoto Chaen
 *
 */
public class PendingIntentSampleService extends Service {
    private NotificationManager mNM = null;
    private int mTimes = 0;

    private int NOTIFICATION = R.string.my_service_started;

    	/**
    	 * 
    	 * @return
    	 */
    public class PendingIntentSampleServiceBinder extends Binder {
       	PendingIntentSampleService getService() {
       		return PendingIntentSampleService.this;
       	}
    }
    

    @Override
    public void onCreate() {
        mNM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);
        mTimes = 0;
		Toast.makeText(this, getString(R.string.my_service_started), Toast.LENGTH_SHORT).show();
	}

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        Log.i("MyService", "Received start id " + startId + ": " + intent);

        showNotification(intent);
        return START_STICKY;
    }

    @Override
    public void onDestroy() {
        mNM.cancel(NOTIFICATION);

        Toast.makeText(this, R.string.my_service_stopped, Toast.LENGTH_SHORT).show();
    }

    @Override
    public IBinder onBind(Intent intent) {
        return mBinder;
    }

    private final IBinder mBinder = new PendingIntentSampleServiceBinder();

    /**
     * 
     * @param intent
     */
    private void showNotification(Intent intent) {
    	PendingIntent contentIntent = null;
    	
		Intent newIntent = new Intent(this, PendingIntentSampleActivity.class);
    	if (intent != null) {
    		Bundle bundle = intent.getExtras();
    		String action = intent.getAction();
    		if (action != null) {
    			Log.d("MyService", "Action: " + action);
    			if (action.equals(PendingIntentSampleActivity.INTENT_ACTION_SHOW_MESSAGE)) {
    				String message = "called !!";
    				if (bundle != null) {
    					message = bundle.getString(PendingIntentSampleActivity.EXTRA_KEY_MESSAGE);
    					int value = bundle.getInt("VALUE", -1);
    					if (value != -1) {
    						message += "with(" + value +")";
    					}
    				}
    				Toast.makeText(this, message, Toast.LENGTH_LONG).show();
    			}
    		}
    		newIntent.fillIn(intent, Intent.FILL_IN_ACTION | Intent.FILL_IN_DATA | Intent.FILL_IN_CATEGORIES | Intent.FILL_IN_PACKAGE);
    		newIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
    	}
    	
    	// 通知する。
		contentIntent = PendingIntent.getActivity(this, 0, newIntent, PendingIntent.FLAG_UPDATE_CURRENT);
        CharSequence text = getString(R.string.my_service_received_intent, mTimes);
        Bitmap largeIconBitmap = BitmapFactory.decodeResource(getResources(), R.drawable.ic_launcher);
        Notification notification = new Notification.InboxStyle(
    		    new Notification.Builder(this)
    		        .setContentTitle(getString(R.string.notification_title))
    		        .setContentIntent(contentIntent)
    		        .setSmallIcon(R.drawable.ic_launcher)
    		        .setLargeIcon(largeIconBitmap)
        			.setAutoCancel(true))
    		    .addLine(text)
    		    .addLine(getString(R.string.notification_text))
    		    .setSummaryText(getString(R.string.notification_summary))
    		    .build();
    		mNM.notify(NOTIFICATION, notification);
    		mTimes++;
    }
    
}